@@ -1,16 +1,16 @@
-import os
-import re
 import contextlib
-import tempfile
-import shutil
 import glob
 import logging
+import os
+import re
+import shutil
+import tempfile
 
 import gpg
-import gpg.errors
 import gpg.constants.sig.mode
+import gpg.errors
 
-from mini_buildd import config, util, call
+from mini_buildd import call, config, util
 
 LOG = logging.getLogger(__name__)
 
@@ -61,7 +61,7 @@ class GpgmeVerifyFailed(util.HTTPUnauthorized):
             _key, info = util.daemon().public_key_cache.keyinfo(signature.fpr)
             return f"{gpg.errors.GpgError(signature.status).code_str}: {info['key']} ({','.join(info['subkeys'])}): {info['user']}"
         except BaseException as e:
-            LOG.warning(f"Can't get keyinfo for gpgme signature: {e}")
+            LOG.warning("Can't get keyinfo for gpgme signature: %s", e)
             return f"{signature}"
 
     @classmethod
@@ -89,7 +89,7 @@ class BaseGnuPG():
             version_line = version_info[0].split(" ")
             return version_line[2][0:3]
         except BaseException as e:
-            LOG.warning(f"Can't parse GPG flavor: {e}")
+            LOG.warning("Can't parse GPG flavor: %s", e)
             return "unknown"
 
     def __init__(self, home):
@@ -101,11 +101,15 @@ class BaseGnuPG():
                         "--batch"]
         self.context = gpg.Context(home_dir=self.home)
 
-        LOG.debug(f"GPG {self.flavor}: {self.gpg_cmd}")
+        LOG.debug("GPG %s: %s", self.flavor, self.gpg_cmd)
 
     def gen_secret_key(self, template):
-        flavor_additions = {"2.1": "\n%no-protection\n",
-                            "2.2": "\n%no-protection\n"}
+        flavor_additions = {
+            "2.1": "\n%no-protection\n",
+            "2.2": "\n%no-protection\n",
+            "2.3": "\n%no-protection\n",
+            "2.4": "\n%no-protection\n",
+        }
 
         with tempfile.TemporaryFile() as t:
             t.write(template.encode(config.CHAR_ENCODING))
@@ -174,7 +178,7 @@ class BaseGnuPG():
         if os.path.exists(keyring):
             self.gpg_cmd += ["--keyring", keyring]
         else:
-            LOG.warning(f"Skipping non-existing keyring file: {keyring}")
+            LOG.warning("Skipping non-existing keyring file: %s", keyring)
 
     def verify(self, signature, data=None):
         try:
@@ -193,7 +197,7 @@ class BaseGnuPG():
             accept = [gpg.errors.NO_ERROR] + [gpg.errors.KEY_EXPIRED] if accept_expired else []
             accepted, failed = [], []
             for s in result.signatures:
-                LOG.info(f"{name} signed with: {GpgmeVerifyFailed.sig_info(s)}")  # This helps maintaining setup's predefinded APT keys in dist.py
+                LOG.info("%s signed with: %s", name, GpgmeVerifyFailed.sig_info(s))  # This helps maintaining setup's predefinded APT keys in dist.py
                 (accepted if gpg.errors.GpgError(s.status).code in accept else failed).append(s)
 
             if not accepted:
@@ -229,10 +233,7 @@ class BaseGnuPG():
             if os.path.exists(signed_file):
                 os.remove(signed_file)
 
-        # .. attention:: **compat**: Retrying sign call -- workaround for mystery bug (:debbug:`849551`)
-        call.call_with_retry(self.gpg_cmd
-                             + ["--armor", "--textmode", "--clearsign", "--output", signed_file]
-                             + (["--local-user", identity] if identity else []) + [unsigned_file],
+        call.call_with_retry(self.gpg_cmd + ["--armor", "--textmode", "--clearsign", "--output", signed_file] + (["--local-user", identity] if identity else []) + [unsigned_file],
                              retry_max_tries=5,
                              retry_sleep=1,
                              retry_failed_cleanup=failed_cleanup)
@@ -247,6 +248,17 @@ class BaseGnuPG():
 
 
 class GnuPG(BaseGnuPG):
+    r"""
+    GnuPG in "home" from config.ROUTES
+
+    >>> # Test non-interactive key generation
+    >>> gnupg_home = tempfile.TemporaryDirectory(prefix="test-home")
+    >>> LOG.info(f"Testing GnuPG() in %s", gnupg_home.name)
+    >>> config.ROUTES = config.Routes(gnupg_home.name)
+    >>> gnupg = GnuPG("Key-Type: RSA\nKey-Length: 4096\nExpire-Date: 0", "y", "z")
+    >>> gnupg.prepare()
+    """
+
     def update(self):
         self.pub_key = self.get_pub_key()
         self.pub_key_id = self.get_first_sec_key()
@@ -270,7 +282,7 @@ Name-Email: {email}
     def remove(self):
         if os.path.exists(self.home):
             shutil.rmtree(self.home)
-            LOG.info(f"GnuPG setup removed: {self.home}")
+            LOG.info("GnuPG setup removed: %s", self.home)
 
     def get_pub_key(self, identity=None):
         return super().get_pub_key("mini-buildd")
@@ -325,9 +337,9 @@ class PublicKeyCache(TmpGnuPG):
         for keyring in glob.glob("/usr/share/keyrings/*-archive-*.gpg"):
             try:
                 call.Call(self.gpg_cmd + ["--import", keyring]).check()
-                LOG.debug(f"PublicKeyCache: Imported {keyring}")
+                LOG.debug("PublicKeyCache: Imported %s", keyring)
             except BaseException as e:
-                LOG.warning(f"PublicKeyCache: Failed to import {keyring}: {e}")
+                LOG.warning("PublicKeyCache: Failed to import %s: %s", keyring, e)
 
     @classmethod
     def key2id(cls, key):
@@ -350,7 +362,7 @@ class PublicKeyCache(TmpGnuPG):
                 if not keyserver:
                     raise util.HTTPBadRequest(f"Key '{key_id}' not in installed archive keys, and no keyserver configured")
                 call.call_with_retry(self.gpg_cmd + ["--armor", "--keyserver", keyserver, "--recv-keys", key_id], retry_max_tries=5, retry_sleep=5, timeout=self.KEYSERVER_RECV_TIMEOUT)
-                LOG.debug(f"PublicKeyCache: '{key_id}' imported from key server '{keyserver}'.")
+                LOG.debug("PublicKeyCache: '%s' imported from key server '%s'.", key_id, keyserver)
 
             # Update pub keys
             pub_keys = self.get_pub_keys()
